unit UAppOptions;

interface

uses Classes, SysUtils, stdCtrls, UGenDeclarations, ActnList;

type
  {---------------------------------------}
  TAppOptions = class(TOptionClass)     // general application options
  protected
    FSettingsChanged: boolean;          // flag to indicate changed settings
    FCaseSensReplace: boolean;          // Case sensitive flag for highlighting
    FCaseCheckBox: TCheckBox;           // control for case sensitive searches
    FSettingsAction: TSettingsaction;   // action to enable/disable depending
    procedure SetCaseSens(Flag: boolean);
    procedure SetCaseCheckBox(Check: TCheckBox);
  public
    constructor Create(Action: TSettingsAction); overload;
    procedure UpdateFromUI;             // read values from controls
    procedure LoadFromStream(ReadBuffer: TMemoryStream); override;
    procedure SaveToStream(OutputBuffer: TMemoryStream); override;
    destructor free; override;
  published
    property CaseSensReplace: boolean read FCaseSensReplace write SetCaseSens;
    property CaseCheckBox: TCheckBox write SetCaseCheckBox;
  end;


  (***********************************************************************************************)
implementation
(*************************************************************************************************)

constructor TAppOptions.create(Action: TSettingsaction);
begin
  FsettingsChanged := false;            // initialize internal members
  FCaseCheckBox := nil;
  if assigned(Action) then
    FSettingsAction := Action
  else
    FSettingsAction := nil
end;
(*************************************************************************************************)

destructor TAppOptions.free;
begin                                   // empty for now
end;
(*************************************************************************************************)

procedure TAppOptions.UpdateFromUI;
begin
  FCaseSensReplace := FCaseCheckBox.Checked;
end;
(*************************************************************************************************)

procedure TAppOptions.SetCaseSens(flag: boolean); // set the case sensitivity flag
begin                                   // and update associated components
  FCaseSensReplace := flag;
  if Assigned(FCaseCheckBox) then
    if FCaseCheckBox.Checked <> flag then
      FCaseCheckBox.Checked := flag;
end;
(*************************************************************************************************)

procedure TAppOptions.SetCaseCheckBox(Check: TCheckBox);
begin                                   // assigns the checkbox component
  if assigned(check) then
    FCaseCheckBox := Check;
end;
(*************************************************************************************************)

procedure TAppOptions.LoadFromStream(ReadBuffer: TMemoryStream);
begin
  ReadBuffer.Read(FCaseSensReplace, sizeof(FCaseSensReplace)); // read case sensitivity flag
  CaseSensReplace := FCaseSensReplace;
  FSettingsAction.SettingsChanged := false;
end;
(*************************************************************************************************)


procedure TAppOptions.SaveToStream(OutputBuffer: TMemoryStream);
begin                                   // save the flag to the stream
  OutputBuffer.Write(FCaseSensReplace, sizeof(FCaseSensReplace));
end;
(*************************************************************************************************)

end.

