unit UConvertClass;

interface
uses ComCtrls, Dialogs, Classes, Forms, Windows, sysutils, Graphics,
  UAppOptions, UHTMLOptions, UIOOptions, UGenDeclarations, Hyperstr;

type


  TConvertClass = class
  private
    FKeepRunning: boolean;              // flag for interruption while processing
    FUpdateClass: TSettingsAction;      // update settings action (enable/disable as needed)
    FPageOptions: THTMLOptions;         // HTML options
    FAppOptions: TAppOptions;           // application options
    FNameList: TNameHighlight;          // list of names to highlight
    FIOOptions: TIOOptions;             // Input/Output options
    FTotalBar: TProgressBar;            // Progressbar to display total progress
    FCurrentBar: TProgressBar;          //     -//-  for the current file only
    FDisplayList: TListView;            // ListView to show conversion success/failure
    FRelImagesDir: string;              // holds the string for relative images dir in HTML
    HTMLPage: TStringList;              // buffer for reading and processing files
    function ConvertSingleFile(const InFile, Outfile: string): TioStatus; // process a single file
    function GetHTMLName(const Infile: string): string; // construct file name for output
    function HighlightFile(const infile, outFile: string): boolean; // highlinghts a single file
    // highlights a single line
    procedure highLight(const OrigStr: string; var FrontStr, BackStr: string);
    procedure fixPaths;                 // add a '\' in front of paths if needed
  public                                // and remove double quotes from LFN paths
    constructor Create(UpdateClass: TSettingsAction); overload;
    procedure AssignControls(TotalBar, CurrentBar: Tprogressbar; DisplayList: TListView);
    procedure AssignOptionObjects(PageOptions: THTMLOptions; AppOptions: TAppOptions;
      NameList: TNameHighlight; IOOptions: TIOOptions);
    function ConvertFiles(var strMessage: string): boolean; // starts the conversion process
    procedure stop;                     // stops conversion
    destructor free;
  published
    property TotalBar: TProgressBar write FTotalBar; // UI control to update during conversion
    property CurrentBar: TProgressBar write FCurrentBar;
    property DisplayList: TListView write FDisplayList;
  end;
  {---------------------------------------}


implementation

(***************************************************************************************************)
function ParseSlashes(const Source: string; DestTree: TStringList): integer; forward;
(***************************************************************************************************)

function min(a, b: integer): integer;   // no comment :)
begin                                   // there is one in the Math unit for those of you with D.Pro +
  if a < b then
    result := a
  else
    result := b;

end;
(***************************************************************************************************)

function ColorToHtml(Color: TColor): string; // By Ramon Prick, on the Borland Delphi ngs.
var                                     // converts internal color representation to HTML
  tempstr: string;                      // color code
begin
  tempstr := IntToHex(ColorToRGB(Color), 6);
  Result := '#' + Copy(tempstr, 5, 2) + Copy(tempstr, 3, 2) + Copy(tempstr, 1, 2);

end;
(******************************************************************************************************)

function AbsToRelPath(const RootPath, DestPath: string): string; // find the relative path from the root
var                                     // path to the destination path
  rootCount: integer;                   // (ie 'c:\windows\command' and 'c:\windows\system' ->
  DestCount: integer;                   // '..\system'. Paths must be on the same drive.
  Index: integer;                       // Untested on UNC paths but should work there too.
  RootTree: tstringlist;
  DestTRee: tstringlist;
  FinalPath: string;
  done: boolean;
begin
  if RootPath[1] <> DestPath[1] then    // MUST be on the same drive
    Result := DestPath
  else if upperCase(RootPath) = upperCase(Destpath) then
    result := ''                        // empty if same path for root and destination
  else
    begin
      finalpath := '';
      RootTree := TStringList.Create;   // keep two trees for the paths
      DestTRee := TStringList.Create;
      RootCount := ParseSlashes(RootPath, RootTree); // parse the paths into tree elements
      DestCount := ParseSlashes(DestPath, DestTree);
      index := 0;
      done := false;
      while (index < min(RootTree.count, DestTree.count)) and not done do
        if RootTree.Strings[index] = DestTree.Strings[index] then
          begin
            RootTree.Delete(index);     // delete common elements to make paths shorter
            DestTree.Delete(index);
            inc(index);
          end
        else
          done := true;
      RootTree.Free;                    // root tree not needed anymore
      dec(RootCount, index + 1);        // (just the number of \ in it)
      dec(DestCount, index + 1);        // correct for deleted common elements
      for index := 0 to RootCount - 1 do // add as many ../ as needed
        finalpath := '../' + finalPath; // (use '/' not '\' in HTML)
      delete(finalPath, length(FinalPath), 1); // delete the first '/' to avoid duplication
      for index := 0 to DestCount - 1 do // now add the path to the destination directory
        if length(DestTree.Strings[index]) > 0 then // only add non-empty dir names to the path
          finalpath := finalpath + '/' + DestTree.Strings[DestCount - index - 1];
      DestTRee.Free;                    // free the Destination path tree now
      if length(FinalPath) > 0 then     // if all this meant smething (ie
        begin
          if rootCount = 0 then         // delete any leading '/' for subdirs
            delete(finalPath, 1, 1);
          if finalpath[length(finalpath)] <> '/' then
            finalpath := finalpath + '/'; // add a '/' if neede to complete path
        end;
      result := finalPath;
    end;

end;
(***************************************************************************************************)

function ExtractFromSlashes(const SlashStr: string): string;
var                                     // extracts a string from another one where
  index: integer;                       // "white-space" is filled with '\'
  done: boolean;                        // substring is located at the end of the string
begin
  done := false;
  index := length(SlashStr);
  while (index > 0) and not done do
    if SlashStr[index] = '\' then
      done := true
    else
      dec(index);
  result := copy(SlashStr, index + 1, length(SlashStr) - index);

end;
(***************************************************************************************************)

function ParseSlashes(const Source: string; DestTree: TStringList): integer;
var
  lastBSlash: boolean;                  // parses paths and stores them backwars in two
  loopcount: integer;                   // trees (used tstringslists for simplicity)
  tempCount: integer;
  tempstr: string;
begin
  DestTree.Clear;
  LastBSlash := false;
  setlength(tempstr, length(source));   // set the length of the temp string beforehand
  fillchar(tempstr[1], length(tempstr), '\'); // to keep things fast. Fill the string with '\'
  tempCount := length(tempstr);         // (arbitrary constant not found in dir names)
  for loopcount := length(source) downto 1 do // repeat for the length of the path parameter
    if Source[loopcount] = '\' then     // another directory has been found
      begin
        if not lastBSlash then          // skipp '\\' (root on UNC names)
          begin
            lastBSlash := true;         // one slash so far
            DestTree.Add(ExtractFromSLashes(tempstr)); // add the directory to the path tree
            tempcount := length(tempstr);
            fillchar(tempstr[1], length(tempstr), '\'); // reinitilize the temp string to '\'
          end
        else
          break;                        // stop if UNC root is reached
      end
    else if Source[loopCount] <> ':' then // stop if path root is reached
      begin
        tempstr[tempCount] := Source[loopCount]; // else add to the temp string the current char.
        dec(tempCount);
        LastBslash := false;            // last char was not a '\'
      end;
  result := DestTree.Count;

end;
(***************************************************************************************************)

function RemoveQuotes(tempstr: string): string; // opposite of quotedstr(): removes double quotes
begin                                   // (for use with Long File Names (LFN)
  if length(tempstr) > 0 then
    if (tempstr[1] = '"') and (tempstr[length(tempstr)] = '"') then
      begin
        delete(tempstr, 1, 1);
        delete(tempstr, length(tempstr), 1);
      end;
  result := tempstr
end;
(***************************************************************************************************)

function SearchStr(Substr, MainStr: string; caseSensitive: boolean): integer;
begin                                   // does a search for a substring respecting the
  if not CaseSensitive then             // case sensitivity flag
    begin
      UCase(Substr, 1, Length(Substr));
      UCase(MainStr, 1, Length(MainStr));
    end;
  result := hyperstr.ScanFF(MainStr,Substr,1);
end;
(***************************************************************************************************)

constructor TConvertClass.create(UpdateClass: TSettingsAction);
begin
  FKeepRunning := true;                 // processing has not been stopped
  if Assigned(UpdateClass) then
    FUpdateClass := UpdateClass;
  HTMLPage := TStringList.create;

end;
(***************************************************************************************************)

destructor TConvertClass.free;
begin
  fiooptions.clearFilesStatus;          // reinitialize the status of the input files
  if assigned(HTMLPage) then
    HTMLPage.Free;

end;
(***************************************************************************************************)

procedure TConvertClass.stop;
begin
  FKeepRunning := false;                // flag to indicate that conversion must be stopped

end;
(***************************************************************************************************)

procedure TconvertClass.AssignControls(TotalBar, CurrentBar: Tprogressbar; DisplayList: TListView);
begin
  if assigned(TotalBar) then            // assign screen controls to be updated
    FTotalBar := TotalBar;
  if assigned(CurrentBar) then
    FCurrentBar := CurrentBar;
  if assigned(DisplayList) then
    FDisplayList := DisplayList;

end;
(***************************************************************************************************)

procedure TconvertClass.AssignOptionObjects(PageOptions: THTMLOptions; AppOptions: TAppOptions;
  NameList: TNameHighlight; IOOptions: TIOOptions);
begin
  if assigned(PageOptions) then         // assign the option objects holding the current
    FPageOptions := PageOptions;        // profile
  if assigned(AppOptions) then
    FappOptions := AppOptions;
  if assigned(NameList) then
    FNameList := NameList;
  if assigned(IOOptions) then
    FIOOptions := IOOptions;

end;
(***************************************************************************************************)

procedure TConvertClass.FixPaths;
var
  index: integer;
begin
  if assigned(FIOOptions) then
    with FIOOptions do
      begin
        if HTMLDir[length(HTMLDir)] <> '\' then // add any necessary '\'
          HTMLDir := HTMLDir + '\';
        if ImagesDir[Length(ImagesDir)] <> '\' then
          ImagesDir := ImagesDir + '\';
      end;
  if assigned(FPageOptions) then        // and remove double quotes
    with FPageOptions do                // from all relevant strings in the configuration
      BackImage := removequotes(BackImage);
  if assigned(FNameList) then
    if FNameList.Count > 0 then
      for index := 0 to FNameList.count - 1 do
        with THighlightInfo(FNameList.Objects[index]) do
          Himage := removeQuotes(Himage);

end;
(***************************************************************************************************)

function TConvertclass.ConvertFiles(var strMessage: string): boolean;
var
  FileIndex: integer;                   // Converts the files in the input list to HTML
  OutResult: TioStatus;
  currFile: string;
  totalStatus: boolean;
  SkipExistingFlag,
    OverwritePromptFlag,
    Settingschanged: boolean;
begin
  Settingschanged := FUpdateClass.SettingsChanged;
  FixPaths;                             // make paths HTML friendly
  FKeepRunning := true;                 // not-suspended flag;
  TotalStatus := true;
  FIOOptions.clearFilesStatus;          // clear any previois results from the file list
  FileIndex := 0;
  // can only proceed if all configuration objects have been assigned
  if assigned(FIOOPtions) and assigned(FAppOptions) and Assigned(FPageOptions) then
    if FIOOptions.InputFiles.Count > 0 then // continue if there's anything to process
      begin
        SkipExistingFlag := FIOOptions.SkipExisting; // get flags for file IO
        OverWritePromptFlag := FIOOPtions.OverwritePrompt;
        with FIOOptions do
          if FPageOptions.IsImagesDirRelative then
            FRelImagesDir := AbsToRelPath(HTMLDir, ImagesDir) // convert paths if needed
          else
            FRelImagesDir := ImagesDir;
        FTotalBar.Max := FIOOptions.InputFiles.Count; // set up controls to be updated
        // keep running until all files have been processed or user signaled to stop conversion
        while (FileIndex < FIOOptions.InputFiles.Count) and FKeepRunning do
          begin
            Application.ProcessMessages;
            CurrFile := GetHTMLName(FIOOptions.InputFiles.Strings[fileIndex]); // get output filename
            // and try to convert the file to HTML
            OutResult := ConvertSingleFile(FIOOptions.InputFiles.Strings[fileIndex], CurrFile);
            if not FKeepRunning then
              OutResult := ioCanceled;  // update status if interrupted
            // update status object (which will update the screen controls)
            FIOOptions.ConvResults[FileIndex] := OutResult;
            // update final result
            TotalStatus := TotalStatus and (OutResult = ioConverted) or (OutResult = ioOverwritten);
            inc(fileIndex);             // next input file
            FTotalBar.StepIt;           // update screen progressBar control
          end;
        FTotalBar.Position := 0;        // clear progress bar when finished
        FIOOptions.SkipExisting := SkipExistingFlag; // restore old values to I/O options object
        FIOOPtions.OverwritePrompt := OverWritePromptFlag;
      end;
  if not FKeepRunning then
    strmessage := 'Canceled by user'
  else if not TotalStatus then
    strMessage := 'One or more files were not converted';
  result := totalstatus;
  FUpdateClass.SettingsChanged := SettingsChanged;

end;
(***************************************************************************************************)

function TConvertClass.GetHTMLName(const Infile: string): string;
begin                                   // returns the output file name based on the paths
  result := FIOOptions.HTMLDir + ChangeFileExt(ExtractFileName(infile), '.htm'); // in the profile

end;
(***************************************************************************************************)

function TConvertClass.ConvertSingleFile(const InFile, OutFile: string): TioStatus;
var
  canSave: boolean;                     // does single file processing
  UserInput: integer;                   // manages IO operations
begin
  result := ioNone;
  canSave := false;
  if fileExists(OutFile) then           // handle existing file cases
    begin                               // based on the options set in the profile objects
      if FIOOptions.SkipExisting then
        result := iofileExists
      else if FIOOptions.OverwritePrompt then
        begin
          Result := ioOverWritten;      // prompt user before overwriting if needed
          UserInput := MessageDlg('Do you want to replace the existing file?',
            mtConfirmation, [mbYes, mbNo, mbNoToAll, mbYesToAll], 0);
          case UserInput of
            IDYes: canSave := true;     // update overite and prompt flags based
            IDNoToAll:
              begin                     // on user response :
                FIOOptions.SkipExisting := true; // "No to all" = skip existing files
                result := ioUserSkipped;
              end;
            IDYesToAll:
              begin                     // "Yes to all" = No overwrite prompt
                canSave := true;
                FIOOPtions.OverwritePrompt := false;
              end;
            IdNo: result := ioUserSkipped; // skip current file only
            else
              canSave := false;
          end;
        end
      else
        begin
          canSave := true;              // if no prompt flag is set
          result := ioOverWritten;      // just overwrite the file without asking
        end
    end
  else
    CanSave := true;                    // if file does not exist, we can write safely
  if CanSave then
    if HighlightFile(InFile, OutFile) then // do the actual highlighting (conversion to HTML)
      begin
        if result <> ioOverwritten then
          result := ioConverted
      end
    else
      result := ioReadError;            // could not read input file for some reason

end;
(******************************************************************************************************)

function TConvertClass.HighlightFile(const inFile, OutFile: string): boolean;
var                                     // converts a file to HTML based on profile options
  LoopIndex: integer;
  StopListen: integer;                  // get user input every once in a while
  tempstr,
  frontstr,
    backstr: string;
  HTMLHeader: string;
begin
  if not fileExists(InFile) then        // check to see if input file exists first
    result := false
  else
    try
      Copyfile(pchar(FPageOptions.BackImage), // copy image files to pecified images directory
        pchar(FIOOptions.ImagesDir + extractfilename(FPageOptions.BackImage)), false);
      for LoopIndex := 0 to FNameList.Count - 1 do // do not overwrite any files in this case
        if assigned(FNameList.objects[LoopIndex]) then // copy images used to mark names in HTML
          begin
            frontstr := THighlightInfo(FNameList.objects[LoopIndex]).himage;
            CopyFile(PChar(frontstr), pchar(FIOOPtions.ImagesDir + extractfilename(frontstr)), false);
          end;
      try
        HTMLPage.Clear;                 // use the same object over to keep mem. usage low
        with FPageOptions do            // fill in the headers for HTML along with basic color
          begin                         // options
            HTMLHeader := '<HTML><META name="author" content="' + Author + '">' + #13 +
              '<TITLE>' + Title + '</TITLE></HEAD>' + #13 +
              '<BODY BGCOLOR="' + ColorToHTML(BackGroundCol) + '"' + #13 +
              'TEXT="' + ColorToHTML(NormalText) + '"' + #13 +
              'LINK="' + ColorToHTML(NormalLink) + '"' + #13 +
              'VLINK="' + ColorToHTML(VisitedLink) + '"' + #13;
            if length(BackImage) > 0 then // background image
              HTMLHeader := HTMLHeader + 'BACKGROUND="' + FRelImagesDir + extractfilename(BackImage);
            // any user entered HTML goes here
            HTMLHeader := HTMLHeader + '"><BR> ' + #13 + CustomFront + '<BR>';
          end;
        HTMLPage.LoadFromFile(inFile);  // load the input file
        HTMLPage.Insert(0, HTMLHeader);
        result := true;
        LoopIndex := 1;                 // skip header lines
        FCurrentBar.position := 0;      // update screen progressBars
        FCurrentBar.Max := TimeSlice;
        if HTMLPage.count > 0 then      // continue if file is not empty
          begin
            StopListen := HTMLPage.count div TimeSlice; // don't get user input on every iteration
            if StopListen < StoppingTime then // to keep processing speed up
              StopListen := StoppingTime;
            while FKeepRunning and (LoopIndex < HTMLPage.count) do
              begin
                if LoopIndex mod StopListen = 0 then
                  begin
                    FCurrentBar.StepIt; // update screen controls after processing the line
                    Application.ProcessMessages; // check for stop flag from user
                  end;
                if FPageoptions.BrokenLongLines then // collapse two adjacent lines if the first one in
                  // too long (adjusting for line wrapping)
                  if length(HTMLPage.Strings[LoopIndex]) > LineLength then
                    with HTMLPage do
                      begin
                        Strings[LoopIndex] := Strings[LoopIndex] + Strings[LoopIndex + 1];
                        Delete(LoopIndex + 1);
                        FCurrentBar.Max := FCurrentBar.max - 1; // adjust screen control too
                      end;
                Highlight(HTMLPage.Strings[LoopIndex], frontStr, BackStr);
                tempstr := HTMLPage.Strings[LoopIndex];
                with HTMLPage do
                  Strings[LoopIndex] := frontstr + tempstr + backstr + '<BR>';
                inc(LoopIndex);
              end;
          end;
        HTMLPage.Add('<BR>' + FPageOptions.CustomEnd); // add any custom HTML here
        HTMLPage.Add('<!-- ' + ProgMessage + ' -->'); // and close the HTML body
        HTMLPage.Add('</BODY> </HTML>');
        if not FKeepRunning then        // return false if user canceled operation
          result := false;
        FCurrentBar.position := 0;      // reset screen control
        HTMLPage.SaveToFile(OutFile);   // and save the converted file
      finally
        HTMLPage.Clear;                 // free the loaded file
      end;
    except
      result := false;                  // return false if input file does not exist
    end;

end;
(******************************************************************************************************)

procedure TconvertClass.highLight(const OrigStr: string; var FrontStr, BackStr: string);
var                                     // highlights a single line from the input file
  index: integer;
  found: boolean;
  HighlightInfo: THighlightInfo;
  ImageStr,
    ThisName: string;
begin
  found := false;
  index := 0;
  FrontStr := '';
  BackStr := '';
  if assigned(FNameList) then           // check if there is anything to search for first
    while not found and (index < FNameList.Count) and FkeepRunning do
      begin
        HighlightInfo := THighlightInfo(FNameList.objects[index]); // get the highlight information
        if not assigned(HighlightInfo) then // if it has not been created
          HighlightInfo := THighlightInfo.Create; // then create it (sets up default values)
        with highlightInfo do
          begin
            if Himage <> '' then
              imagestr := FRelImagesDir + extractfilename(Himage);
            thisName := FNameList.Strings[index]; // + speakStr;
            if searchStr(ThisName, OrigStr, FAppOptions.CaseSensReplace) > 0 then
              begin                     // searches in first come first served fashion
                found := true;          // stop if already found
                // create HTML tages and assign them to the string
                FrontStr := '<FONT color="' + ColorToHTML(Hcolor) + '"> ' + #13 +
                  HCustomFrontHTML + #13;
                if length(ImageStr) > 0 then
                  FrontStr := FrontStr + '<img src="' + imagestr + '">';
                BackStr := #13 + HCustomEndHTML + ' </FONT>';
              end;
          end;
        inc(index);                     // next name to search for
      end;

end;
(******************************************************************************************************)
end.

