unit UGenDeclarations;

interface
uses Classes, SysUtils, ActnList;

{---------------------------------------}
const                                             // global constants
  IDNoToAll = 9;                                  // Message box "No to all" id
  IDYesToAll = 10;                                // Message box "Yes to all" id

  TimeSlice = 20;
  Stoppingtime = 100;

  TagNameColor = 0;                               // used to determine which control requested
  TagPageColor = 1;                               // the PickColor dialog box
  TagTextColor = 2;
  TagLinkColor = 3;
  TagVisLinkColor = 4;
  TagInvalid = -1;

  DefBackgroundConst = $400040;                   // dark grey
  DefTextConst = $FFFFFF;                         // white
  DefLinkConst = $00FFFF;                         // cyan
  DefVisitedLinkConst = $FF0000;                  // red

  MaxPathLength = 255;
  MaxMRUConst = 20;                               // size of the MRU list
  LineLength = 76;                                // line length (for joinning broken lines)
  ConfigExtConst = '.cfg';                        // default profile file extention
  //  SpeakStr = ':';                             // chat indicator in dumps
  ProgMessage = 'Created by Text2HTML';         // message to apear in HTML
  DefOpStatus = 'Ready';                          // default IO operation status = no IO yet
  {---------------------------------------}

type

  THTMLColor = integer;
  TNameHighlight = TStringList;                   // list of names to highlight and highlight properties
  EGenException = class(Exception);               // (stored in the TNameHighLight.objects list)
  ESizeException = class(Exception);

  {---------------------------------------}
  TioStatus = (ioReady, ioConverted, IoOverwritten, ioUserSkipped, ioReadError, // file ops. result
    ioFileExists, ioWriteError, ioNone, ioCanceled);

  {---------------------------------------}
  IoResClass = class                              // container class for filoe IO operation results
    status: TioStatus;
    function ResToStr(theresult: TioStatus): string;
    function StrResult: string;
    constructor create; overload;
  end;

  {---------------------------------------}
  TSettingsAction = class                         // wrapper class for the Update Action
  private                                         // (enable it on settings changed)
    FSettingsAction: Taction;                     // action to be enabled/disabled
    procedure setSettingsFLag(flag: boolean);
    function GetSettingsFlag: boolean;
  public
    constructor create(Action: TAction); overload;
  published
    property SettingsChanged: boolean read GetSettingsFlag write SetSettingsFlag;
  end;

  {---------------------------------------}
  TOptionClass = class                            // parent for configuration classes
  protected
    //    constructor create; virtual; abstract;        // methods only implemented in descendants
    procedure LoadFromStream(ReadBuffer: TMemoryStream); virtual; abstract;
    procedure SaveToStream(OutputBuffer: TMemoryStream); virtual; abstract;
    destructor free; virtual; abstract;           //
  end;

  {---------------------------------------}

  THighLightInfo = class                          // Class for highlighting properties for each entry
    HColor: THTMLColor;
    HImage: string;                               // image apearing left of text
    HCustomFrontHTML,                             // any custom formatting goes here
    HCustomEndHTML: string;
    constructor create; overload;
  end;
  {---------------------------------------}

var
  DefBackground: integer = DefBackgroundConst;
  DefText: integer = DefTextConst;
  DefLink: integer = DefLinkconst;
  DefVisitedLink: integer = DefVisitedLinkConst;

  MaxMRU: integer = MaxMRUConst;
  ConfigExt: string = ConfigExtConst;
  (****************************************************************************************************)
implementation
(******************************************************************************************************)

constructor ThighlightInfo.create;
begin
  inherited;
  HColor := DefText;                              // assign default value for color
  HImage := '';                                   // and initialize strings to empty
  HCustomFrontHTML := '';
  HCustomFrontHTML := '';
end;
(******************************************************************************************************)

constructor tSettingsAction.create(Action: TAction);
begin
  if assigned(Action) then
    FSettingsAction := Action
  else
    FSettingsAction := nil
end;
(******************************************************************************************************)

procedure TsettingsAction.setSettingsFLag(flag: boolean);
begin
  if assigned(FsettingsAction) then
    FSettingsAction.Enabled := flag;
end;
(******************************************************************************************************)

function TsettingsAction.GetSettingsFLag: boolean;
begin
  if assigned(FsettingsAction) then
    result := FSettingsAction.Enabled
  else
    result := true;
end;
(******************************************************************************************************)

constructor IOResClass.create;
begin
  status := ioReady
end;
(******************************************************************************************************)

function IOResClass.ResToStr(theResult: TioStatus): string;
begin                                             // convert an IOresult enumerated type to
  case theResult of                               // a string message for display
    ioReady: result := DefOpStatus;
    ioConverted: result := 'Converted';
    IoOverwritten: result := 'Converted. Overwritten existing file';
    ioUserSkipped: result := 'Not converted. Skipped existing file';
    ioReadError: result := 'Error reading source file';
    ioFileExists: result := 'Not converted. Skipped existing file';
    ioWriteError: result := 'Not converted. Error writing to output file';
    ioNone: result := 'Unknown status';
    ioCanceled: result := 'Canceled by user';
  else
    result := 'Unknown status';
  end;
end;
(******************************************************************************************************)

function IOResClass.StrResult: string;            // return a string message for the current io status
begin
  result := restostr(status);
end;
(******************************************************************************************************)
end.
