unit UHTMLOptions;

interface
uses Classes, StdCtrls, ToolEdit, UGenDeclarations;

type
  {---------------------------------------}
  THTMLOptions = class(TOptionClass)    // General HTML page options
  private
    FSettingsAction: TSettingsAction;
    FBackGroundCol,
      FNormalText,                      // regular text color
      FNormalLink,                      // link color
      FVisitedLink: THTMLColor;         // visited link color
    FBackImage,                         // background image
      FCustomFront,                     // any custom HTML code or text goes here
      FCustomEnd,
      FTitle,                           // page title
      FAuthor: string;                  // author name
    FIsImagesDirRelative,               // flag to store relative paths in HTML
      FBrokenLongLines: boolean;        // highlight long lines as broken
    FcontrolsAssigned: Boolean;
    // list of controls to be updated by this class
    FTitleEdit,                         // page title edit control
      FAuthEdit: TEdit;                 // page author edit control
    FBackImgEdit: TFileNameEdit;        // background image edit control
    FRelImgCheck,                       // relative image paths checkbox control
      FBrokenLinesCheck: TCheckBox;     // Merge broken lines checkbox control
    FfrontHTMLMemo,                     // custom HTML at top of page memo control
      FEndHTMlMemo: TMemo;              // custom HTML at bottom of page memo control
    FBackColLabel,                      // page background color label control
      FTextColLabel,                    // page text color label control
      FLinkcolLabel,                    // page link color label control
      FVisLinkcolLabel: Tlabel;         // page visited link color label control

    procedure SetBackGround(value: THTMLcolor);
    procedure SetTextCol(value: THTMLcolor);
    procedure SetLinkCol(value: THTMLcolor);
    procedure SetVisLinkCol(value: THTMLcolor);
    procedure SetBackImg(value: string);
    procedure SetTitle(value: string);
    procedure SetAuth(value: string);
    procedure SetFrontHTML(value: string);
    procedure SetEndHTML(value: string);
    procedure SetRelImagesDir(value: boolean);
    procedure SetbrokenLongLines(value: boolean);
  public
    constructor create(Action: TSettingsAction); overload; // (ie. color next line the same way)
    procedure UpdateControls;           // update all assigned screen controls
    procedure UpdateFromUI;             // read values from the assigned controls
    procedure LoadFromStream(ReadBuffer: TMemoryStream); override;
    procedure SaveToStream(OutputBuffer: TMemoryStream); override;
    // assign screen controls to the internal ones to be updated
    procedure assignControls(TitleEdit, AuthEdit: TEdit; BackImgEdit: TFileNameEdit;
      RelImgCheck, BrokenLinesCheck: TCheckBox; frontHTMLMemo, EndHTMlMemo: TMemo;
      BackColLabel, TextColLabel, LinkcolLabel, VisLinkcolLabel: Tlabel);

    destructor free; override;
  published
    property BackGroundCol: THTMLColor read FBackGroundCol write SetBackGround;
    property NormalText: THTMLColor read FNormalText write SetTextCol;
    property NormalLink: THTMLColor read FNormalLink write SetLinkCol;
    property VisitedLink: THTMLColor read FVisitedLink write SetVisLinkCol;
    property BackImage: string read FBackImage write SetBackImg;
    property CustomFront: string read FCustomFront write FCustomFront;
    property CustomEnd: string read FCustomEnd write FCustomEnd;
    property Title: string read FTitle write Settitle;
    property Author: string read FAuthor write SetAuth;
    property IsImagesDirRelative: boolean read FIsImagesDirRelative write FIsImagesDirRelative;
    property BrokenLongLines: boolean read FBrokenLongLines write FBrokenLongLines;
  end;


  (*********************************************************************************************)
implementation
(***********************************************************************************************)

constructor THTMLOptions.create(Action: TsettingsAction);
begin
  FBackGroundCol := DefBackGround;      // initialize HTML attributes to default ones
  FNormalText := DefText;
  FNormalLink := DefLink;
  FVisitedLink := DefVisitedLink;
  FBackImage := '';
  FCustomFront := '';
  FCustomEnd := '';
  FTitle := '';
  FIsImagesDirRelative := true;         // store relative image paths by default
  FcontrolsAssigned := False;
  FTitleEdit := nil;                    // initially blank out controls to be updated
  FAuthEdit := nil;
  FBackImgEdit := nil;
  FRelImgCheck := nil;
  FBrokenLinesCheck := nil;
  FfrontHTMLMemo := nil;
  FEndHTMlMemo := nil;
  FBackColLabel := nil;
  FTextColLabel := nil;
  FLinkcolLabel := nil;
  FVisLinkcolLabel := nil;
  if assigned(Action) then
    FSettingsAction := Action
  else
    FsettingsAction := nil;
end;
(***********************************************************************************************)

destructor THTMLOptions.free;
begin                                   // empty for now
end;
(***********************************************************************************************)

procedure THTMLOptions.assignControls(TitleEdit, AuthEdit: TEdit; BackImgEdit: TFileNameEdit;
  RelImgCheck, BrokenLinesCheck: TCheckBox; frontHTMLMemo, EndHTMlMemo: TMemo;
  BackColLabel, TextColLabel, LinkcolLabel, VisLinkcolLabel: Tlabel);
begin
  try
    FTitleEdit := TitleEdit;            // assign the screen controls to be updated
    FAuthEdit := AuthEdit;
    FBackImgEdit := BackImgEdit;
    FRelImgCheck := RelImgCheck;
    FBrokenLinesCheck := BrokenLinesCheck;
    FfrontHTMLMemo := frontHTMLMemo;
    FEndHTMlMemo := EndHTMlMemo;
    FBackColLabel := BackColLabel;
    FTextColLabel := TextColLabel;
    FLinkcolLabel := LinkcolLabel;
    FVisLinkcolLabel := VisLinkcolLabel;
    UpdateControls;
    FcontrolsAssigned := true;
  except
    FcontrolsAssigned := False;
  end;
end;
(***********************************************************************************************)

procedure THTMLOptions.UpdateControls;  // setter for background color
begin
  if FControlsAssigned then
    begin
      SetBackGround(FBackGroundCol);    // update the screen controls
      SetTextCol(FNormalText);
      SetLinkCol(FNormalLink);
      SetVisLinkCol(FVisitedLink);
      SetBackImg(FBackImage);
      SetTitle(FTitle);
      SetAuth(Fauthor);
      SetFrontHTML(FCustomFront);
      SetEndHTML(FCustomEnd);
      SetRelImagesDir(FIsImagesDirRelative);
      SetbrokenLongLines(FBrokenLongLines);
    end;
end;
(***********************************************************************************************)

procedure THTMLOptions.UpdateFromUI;
begin
  if FcontrolsAssigned then
    begin
      FBackGroundCol := FBackColLabel.Color; // Whenever a screen object is changed
      FNormalText := FTextColLabel.Color; // the memory objects need to be updated
      FNormalLink := FLinkColLabel.Color; // screen objects are not saved to disk
      FVisitedLink := FVisLinkColLabel.Color;
      FIsImagesDirRelative := FRelImgCheck.Checked;
      FBrokenLongLines := FBrokenLinescheck.Checked;
      FBackImage := FBackImgEdit.Text;
      FTitle := FTitleEdit.Text;
      Fauthor := FAuthEdit.Text;
      FCustomFront := FFrontHTMLMemo.Text;
      FCustomEnd := FEndHTMLMemo.Text;
    end;
end;
(***********************************************************************************************)

procedure THTMLOptions.setBackGround(Value: THtMLColor); // setter for background color
begin
  FBackGroundCol := value;
  if assigned(FBackColLabel) then
    FBackColLabel.Color := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetTextCol(Value: THtMLColor); // setter for text color
begin
  FNormalText := value;
  if assigned(FTextColLabel) then
    FTextColLabel.Color := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetLinkCol(Value: THtMLColor); // setter for link color
begin
  FNormalLink := value;
  if assigned(FLinkcolLabel) then
    FLinkcolLabel.Color := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetVisLinkCol(Value: THtMLColor); // setter for visited link color
begin
  FVisitedLink := value;
  if assigned(FLinkcolLabel) then
    FVisLinkcolLabel.Color := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.setBackIMG(Value: string); // setter for background image file
begin
  FBackImage := value;
  if assigned(FBackImgEdit) then
    FBackImgEdit.text := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetTitle(Value: string); // setter for page title
begin
  FTitle := value;
  if assigned(FBackImgEdit) then
    FTitleEdit.text := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetAuth(Value: string); // setter for page author
begin
  FAuthor := value;
  if assigned(FAuthEdit) then
    FAuthEdit.text := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetFrontHTMl(Value: string); // setter for custom HTML
begin
  FCustomFront := value;
  if assigned(FfrontHTMLMemo) then
    FfrontHTMLMemo.text := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetEndHTML(Value: string); // setter for custom HTML
begin
  FCustomEnd := value;
  if assigned(FfrontHTMLMemo) then
    FEndHTMlMemo.text := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetRelImagesDir(Value: boolean); // setter for relative image paths flag
begin
  FIsImagesDirRelative := value;
  if assigned(FRelImgCheck) then
    FRelImgCheck.checked := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.SetbrokenLongLines(Value: boolean); // setter for merge broken lines flag
begin
  FBrokenLongLines := value;
  if assigned(FBrokenLinesCheck) then
    FBrokenLinesCheck.checked := value;
end;
(***********************************************************************************************)

procedure THTMLOptions.LoadFromStream(ReadBuffer: TMemoryStream);
var
  strlength: longint;
begin
  // first 4 bytes hold complete file size
  ReadBuffer.Read(FBackgroundCol, sizeof(FBackgroundCol)); // read some HTML color values
  ReadBuffer.Read(FNormalText, sizeof(FNormalText));
  ReadBuffer.Read(FNormalLink, sizeof(FnormalLink));
  ReadBuffer.Read(FVisitedLink, sizeof(FVisitedLink));

  SetBackGround(FBackgroundCol);        // update the screen controls too
  SetTextCol(FNormalText);
  SetLinkCol(FNormalLink);
  SetVisLinkCol(FVisitedLink);

  ReadBuffer.Read(FBrokenLongLines, sizeof(FBrokenLongLines)); // read flags
  ReadBuFfer.Read(FIsImagesDirRelative, sizeof(FIsImagesDirRelative));

  SetRelImagesDir(FIsImagesDirRelative); // update screen controls
  SetbrokenLongLines(FBrokenLongLines);

  ReadBuffer.read(strlength, sizeof(strlength)); // Read length of following string.
  setlength(FBackImage, strlength);     // Set the strings (background image) length,
  ReadBuffer.Read(FBackImage[1], strlength); // and read it.

  SetBackImg(FBackImage);               // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength)); // Repeat above for :
  setlength(FCustomFront, strlength);
  ReadBuffer.Read(FCustomFront[1], strlength); // Custom HTML,

  SetFrontHTML(FCustomFront);           // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength));
  setlength(FCustomEnd, strlength);
  ReadBuffer.Read(FCustomEnd[1], strlength);

  SetEndHTML(FCustomEnd);               // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength));
  setlength(FTitle, strlength);
  ReadBuffer.Read(FTitle[1], strlength); // page title,

  SetTitle(FTitle);                     // update screen controls

  ReadBuffer.Read(strlength, sizeof(strlength));
  setlength(Fauthor, strlength);
  ReadBuffer.Read(Fauthor[1], strlength); // and author name.

  SetAuth(Fauthor);                     // update screen controls
end;
(***********************************************************************************************)

procedure THTMLOptions.SaveToStream(OutputBuffer: TMemoryStream);
var
  strlength: longint;
begin
  OutputBuffer.Write(FBackgroundCol, sizeof(FBackgroundCol)); // save HTML color attributes
  OutputBuffer.Write(FNormalText, sizeof(FNormalText));
  OutputBuffer.Write(FNormalLink, sizeof(FNormalLink));
  OutputBuffer.Write(FVisitedLink, sizeof(FVisitedLink));

  Outputbuffer.Write(FBrokenLongLines, sizeof(FbrokenLongLines)); // save flags
  OutputBuFfer.Write(FIsImagesDirRelative, sizeof(FIsImagesDirRelative));

  strlength := length(FBackImage);
  OutputBuffer.Write(strlength, sizeof(strlength)); // save text length for background image
  OutputBuffer.Write(FBackImage[1], strlength); // save background image filename and path

  strlength := length(FCustomFront);
  OutputBuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FCustomFront[1], strlength);
  strlength := length(FCustomEnd);
  OutputBuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FCustomEnd[1], strlength); // save custom HTML

  strlength := length(FTitle);
  OutputBuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FTitle[1], strlength); // save HTML page title

  strlength := length(FAuthor);
  OutputBuffer.Write(strlength, sizeof(strlength));
  OutputBuffer.Write(FAuthor[1], strlength); // save author name

  strlength := OutputBuffer.position;   // save the current position = stream size
  OutputBuffer.Seek(sizeof(pointer) * 2, soFromBeginning); // go back to the beginning
  OutputBuffer.Write(strlength, sizeof(strlength)); // and save the size descriptor

  OutputBuffer.Seek(strlength, soFromBeginning); // skip back to the end for any further writing
end;
(***********************************************************************************************)
end.

